      subroutine read_pixel_frame(imghead, dimem, pixels, err_code)

C     Read a single pixel frame from a boxed image file and store in a
C     real array. Allowed image file formats are currently restricted to
C     the following
C     
C     mode=0  - char  (1 byte)
C     mode=1  - short (2 bytes)
C     mode=6  - char  (2 bytes)
C     mode=46 - int   (4 bytes)
C
C     Need to make sure that fields in imghead have already been filled
C     before calling this routine. Typically done using call to
C     pif_read_dh and by manually setting imghead%imgnum to point to
C     correct image.
C
C     This routine is adapated from Hong Lin's rpifimag_f and has been
C     modified so as to be easier to maintain and more straightforward
C     to call directly without having to worry about using common blocks.
C
C     imghead   Image header from PIF boxed image file
C     dimem     Number of pixels in image
C     pixels    Array of pixel values
C     err_code  Error code

      implicit none
      include '../include/pifhead.inc'

      type(pifhead_t), intent(in) :: imghead
      integer, intent(in)         :: dimem
      real, intent(out)           :: pixels(dimem)
      integer, intent(out)        :: err_code

      character pixels_buf_byte(dimem)
      integer*2 pixels_buf_int2(dimem)
      integer   i4, k
      real      bkg

C     Initialize err_code to failure. Will be overwritten if successful
      err_code = 1

C     Process byte image
      if (imghead%mode == 0 .or. imghead%mode == 6) then
         call pif_read_byte_image (imghead%luNum, pixels_buf_byte,
     $        imghead%imgOffset, imghead%numCols, imghead%numRows, 1)
         i4 = ichar(pixels_buf_byte(1))
         if(i4 < 0) i4 = i4 + 256
         bkg = real(i4)
         do k=1,dimem
            i4 = ichar(pixels_buf_byte(k))
            if(i4 < 0) i4 = i4 + 256
            pixels(k) = real(i4) - bkg
         enddo
         err_code = 0
      endif

C     Process int*2 images
      if (imghead%mode == 1) then
         call pif_read_short_image (imghead%luNum, pixels_buf_int2,
     $        imghead%imgOffset, imghead%numCols, imghead%numRows, 1)
         i4 = int(pixels_buf_int2(1))
         if(i4 < 0) i4 = i4 + 256*256
         bkg = real(i4)
         do k=1,dimem
            i4 = int(pixels_buf_int2(k))
            if(i4 < 0) i4 = i4 + 256*256
            pixels(k) = real(i4) - bkg
         enddo
         err_code = 0
      endif

C     Process int*4 images
      if (imghead%mode == 46) then
         call pif_read_boxi4(imghead, pixels)
         bkg = real(pixels(1))
         do k=1,dimem
            pixels(k) = pixels(k) - bkg
         enddo
         err_code = 0
      endif

      return
      end subroutine read_pixel_frame
